package com.gic.haoban.manage.service.mq.processor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * desc：处理器
 *
 * @author: YongEn
 * @date: 2022/2/16
 **/
public abstract class AbstractConsumerProcessor<S, R> {
    private static final Logger logger = LoggerFactory.getLogger(AbstractConsumerProcessor.class);

    public void process(S messageParam) {
        logger.info("接收到消费处理, 参数{}", messageParam);
        // 参数解析
        R r = paramParse(messageParam);
        // 执行处理
        doProcess(r);
    }

    /**
     * desc: 参数解析
     *
     * @param s 原数据
     * @return : {@link R}
     * @author : YongEn
     * @date : 2022/2/16
     */
    abstract R paramParse(S s);

    /**
     * desc: 执行处理
     *
     * @param r 解析参数
     * @return :
     * @author : YongEn
     * @date : 2022/2/16
     */
    abstract void doProcess(R r);

}
