package com.gic.haoban.manage.service.service.out.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StoreDetailDTO;
import com.gic.enterprise.api.dto.StoreVo;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.common.utils.UuidUtil;
import com.gic.haoban.communicate.api.service.SyncHaobanToGicServiceApi;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.*;
import com.gic.haoban.manage.api.service.AuditApiService;
import com.gic.haoban.manage.api.service.StaffClerkRelationApiService;
import com.gic.haoban.manage.api.service.StaffDepartmentRelatedApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.api.util.notify.NoticeMessageUtil;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanAuditMapper;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanBatchAuditLogMapper;
import com.gic.haoban.manage.service.entity.TabHaobanAudit;
import com.gic.haoban.manage.service.entity.TabHaobanBatchAuditLog;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by tgs on 2020/2/9.
 */
@Service
public class AuditApiServiceImpl implements AuditApiService {
    private static final Logger logger = LoggerFactory.getLogger(AuditApiServiceImpl.class);

    @Autowired
    private TabHaobanAuditMapper auditMapper;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private StoreService storeService;
    @Autowired
    private TabHaobanBatchAuditLogMapper tabHaobanBatchAuditLogMapper;
    @Autowired
    private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private SyncHaobanToGicServiceApi syncHaobanToGicServiceApi;
    @Autowired
    private StaffClerkRelationApiService staffClerkRelationApiService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;


    @Override
    public Page<AuditDTO> page(Integer auditType, String search, String wxEnterpriseId,
                               String enterpriseId, Integer auditStatus, Integer auditFlag, BasePageInfo pageInfo) {
        //部门关联表修改，服务层调整
        List<String> departmentIds = new ArrayList<>();
        List<String> staffIds = new ArrayList<>();
        List<String> storeIds = new ArrayList<>();
        if (StringUtils.isNotEmpty(search)) {
            //获取企业列表
            List<String> enterpriseIds = new ArrayList<>();
            if (StringUtils.isEmpty(enterpriseId)) {
                List<EnterpriseDetailDTO> enterprises = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
                if (CollectionUtil.isNotEmpty(enterprises)) {
                    List<String> searchEnterpriseIds = enterprises.stream().map(s -> s.getEnterpriseId()).collect(Collectors.toList());
                    enterpriseIds.addAll(searchEnterpriseIds);
                }
            } else {
                enterpriseIds.add(enterpriseId);
            }
            //调gic接口查storeIds
            String[] strArray = new String[enterpriseIds.size()];
            enterpriseIds.toArray(strArray);

            List<StoreVo> list = storeService.queryStoreByName(strArray, search);
            if (CollectionUtil.isNotEmpty(list)) {
                List<String> storeIds1 = list.stream().map(s -> s.getStoreId()).collect(Collectors.toList());
                storeIds.addAll(storeIds1);
            }
            //模糊查staffName
            List<TabHaobanStaff> staffList = staffMapper.listLikeName(search, wxEnterpriseId);
            if (CollectionUtil.isNotEmpty(staffList)) {
                List<String> staffIds2 = staffList.stream().map(s -> s.getStaffId()).collect(Collectors.toList());
                staffIds.addAll(staffIds2);
            }
            if (CollectionUtil.isEmpty(staffIds)) {
                staffIds.add("-999");
            }
            if (CollectionUtil.isEmpty(storeIds)) {
                storeIds.add("-999");
            }
        }
        PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
        com.github.pagehelper.Page<TabHaobanAudit> page = new com.github.pagehelper.Page<TabHaobanAudit>();
        if (CollectionUtil.isNotEmpty(storeIds) && CollectionUtil.isNotEmpty(staffIds)) {
            page = auditMapper.pageForStoreIdsAndstaffIds(auditType, storeIds, staffIds, wxEnterpriseId, enterpriseId, auditStatus, auditFlag);
        } else {
            page = auditMapper.page(auditType, storeIds, staffIds, wxEnterpriseId, enterpriseId, auditStatus, auditFlag);
        }
        if (CollectionUtil.isNotEmpty(page.getResult())) {
            List<String> commitStaffIds = page.getResult().stream().map(s -> s.getCommitStaffId()).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(commitStaffIds)) {
                List<TabHaobanStaff> staffList = staffMapper.listByIds(commitStaffIds);
                Map<String, TabHaobanStaff> map = com.gic.commons.util.CollectionUtil.toMap(staffList, "staffId");
                for (TabHaobanAudit tab : page.getResult()) {
                    String commitStaffId = tab.getCommitStaffId();
                    tab.setCommitStaffName(map.get(commitStaffId) == null ? "" : map.get(commitStaffId).getStaffName());
                    tab.setCommitStaffImg(map.get(commitStaffId) == null ? "" : map.get(commitStaffId).getHeadImg());
                }
            }
        }
        return PageUtil.changePageHelperToCurrentPage(page, AuditDTO.class);
    }

    @Transactional
    @Override
    public String audit(String batchId, String auditId, String auditName, String optStaffId) {
        TabHaobanAudit tab = auditMapper.selectByPrimaryKey(auditId);
        if (tab == null) {
            return "审核项目不存在";
        }
        int auditType = tab.getAuditType();
        tab.setAuditId(auditId);
        tab.setUpdateTime(new Date());
        tab.setAuditName(auditName);
        tab.setAuditStaffId(optStaffId);
        if (auditType == AuditType.STORE_CHANGE.getCode()) {
            //门店信息变更
            String storeId = tab.getCommitStoreId();
            String oldValue = tab.getOldValue();
            String newValue = tab.getNewValue();
            String changeField = tab.getChangeField();
            StoreDTO store = storeService.getStore(storeId);
            if (store == null) {
                return "门店不存在";
            }
            this.editStoreInfo(storeId, changeField, oldValue, newValue);
            //发送消息
            String title = "门店信息修改通知";
            String content = StoreFieldEnum.getValueName(changeField);
            content = "[" + content + "]" + "修改审核通过";
            TabHaobanAudit tab2 = auditMapper.selectByPrimaryKey(auditId);
            String staffId = tab2.getCommitStaffId();
//			this.sendMessageByStaffId(staffId, storeId, title, content);
            //插入log
            if (org.apache.commons.lang.StringUtils.isNotBlank(batchId)) {
                insertBatchLog(batchId, AuditRsultType.success.getCode(), tab);
            }
            this.sendMessageByStaffId(staffId, storeId, false, content, "");
        } else if (auditType == AuditType.CLERK_BIND.getCode()) {
            //门店导购绑定
            String oldValue = tab.getOldValue();
            AuditStaffDTO obj = JSONObject.parseObject(oldValue, AuditStaffDTO.class);
            StaffClerkRelationDTO staffClerkRelation = staffClerkRelationApiService.getByCodeAndEnterpriseId(obj.getClerkCode(), tab.getEnterpriseId());
            String staffId = obj.getStaffId();
            String storeId = tab.getCommitStoreId();
            StaffClerkRelationDTO have = staffClerkRelationApiService.getOneByStoreIdAndStaffId(storeId, staffId);
            if (have != null) {
                tab.setAuditStatus(5);
                tab.setAuditReason("该成员在该门店下已经绑定过一个导购了");
                auditMapper.updateByPrimaryKeySelective(tab);
                if (org.apache.commons.lang.StringUtils.isNotBlank(batchId)) {
                    insertBatchLog(batchId, AuditRsultType.fail.getCode(), tab);
                }
                staffDepartmentRelatedApiService.sendBindAuditMessage(auditId, obj.getStaffId(), obj.getClerkId(), tab.getAuditReason(), true);
                return "该成员在该门店下已经绑定过一个导购了";
            }

            ClerkDTO clerkDTO = clerkService.getClerkByClerkId(obj.getClerkId());
            if (clerkDTO == null) {
                tab.setAuditStatus(5);
                tab.setAuditReason("gic已删除该导购");
                auditMapper.updateByPrimaryKeySelective(tab);
                if (org.apache.commons.lang.StringUtils.isNotBlank(batchId)) {
                    insertBatchLog(batchId, AuditRsultType.fail.getCode(), tab);
                }
                staffDepartmentRelatedApiService.sendBindAuditMessage(auditId, obj.getStaffId(), obj.getClerkId(), tab.getAuditReason(), true);
                return "gic已删除该导购";
            } else {
                if (staffClerkRelation == null) {
                    staffClerkRelation = new StaffClerkRelationDTO();
                }
                staffClerkRelation.setEnterpriseId(tab.getEnterpriseId());
                staffClerkRelation.setClerkCode(obj.getClerkCode());
                staffClerkRelation.setClerkId(obj.getClerkId());
                staffClerkRelation.setStoreId(tab.getCommitStoreId());
                staffClerkRelation.setWxEnterpriseId(tab.getWxEnterpriseId());
                staffClerkRelation.setWxUserId(obj.getWxUserId());
                staffClerkRelation.setStaffId(obj.getStaffId());
                staffClerkRelationApiService.delAndInsert(staffClerkRelation, optStaffId, ChannelCodeEnum.AUDIT_BIND.getCode());
            }
            if (org.apache.commons.lang.StringUtils.isNotBlank(batchId)) {
                insertBatchLog(batchId, AuditRsultType.success.getCode(), tab);
            }
            //同意绑定
            staffDepartmentRelatedApiService.sendBindAuditMessage(auditId, obj.getStaffId(), obj.getClerkId(), "", false);

//			staffDepartmentRelatedApiService.sendClerkBind(obj.getStaffId(), obj.getClerkCode(),storeId, 1, "");

        } else if (auditType == AuditType.CLERK_ADD.getCode()) {
            //门店导购新增
            String oldValue = tab.getOldValue();
            AuditStaffDTO obj = JSONObject.parseObject(oldValue, AuditStaffDTO.class);
            String flag = syncHaobanToGicServiceApi.syncClerkToGicClerkAddNew(tab.getCommitStoreId(), obj.getClerkCode(), obj.getSex(), obj.getClerkName(), obj.getPhoneNumber(), obj.getNationCode(), null);
            if (!"1".equals(flag)) {
                //失败
                tab.setAuditStatus(5);
                tab.setAuditReason(flag);
                auditMapper.updateByPrimaryKeySelective(tab);
                if (org.apache.commons.lang.StringUtils.isNotBlank(batchId)) {
                    insertBatchLog(batchId, AuditRsultType.fail.getCode(), tab);
                }
                return flag;
            } else {
                //成功
                if (org.apache.commons.lang.StringUtils.isNotBlank(batchId)) {
                    insertBatchLog(batchId, 1, tab);
                }
                staffDepartmentRelatedApiService.sendClerkAdd(obj.getClerkName(), obj.getClerkCode(), tab.getCommitStoreId(), 1, "");
            }

        } else if (auditType == AuditType.CLERK_DEL.getCode()) {
            //门店导购删除
            String oldValue = tab.getOldValue();
            AuditStaffDTO obj = JSONObject.parseObject(oldValue, AuditStaffDTO.class);
            syncHaobanToGicServiceApi.delGicClerk(obj.getClerkId());
            if (org.apache.commons.lang.StringUtils.isNotBlank(batchId)) {
                insertBatchLog(batchId, AuditRsultType.success.getCode(), tab);
            }
            //解绑日志
            StaffClerkRelationDTO relationDTO = staffClerkRelationApiService.getByClerkId(obj.getClerkId());
            if (null != relationDTO) {
                staffClerkRelationApiService.pushToBindLog(relationDTO.getStaffId(), optStaffId, BindTypeEnum.UNBIND.getVal(), ChannelCodeEnum.SELF_UNBIND.getCode(), relationDTO.getStaffClerkRelationId());
            }
            staffClerkRelationApiService.delByClerkId(obj.getClerkId());
            staffDepartmentRelatedApiService.sendClerkDel(obj.getClerkName(), obj.getClerkCode(), tab.getCommitStoreId(), 1, "");
        } else if (auditType == AuditType.CLERK_UNBIND.getCode()) {
            //解绑申请，无需审核，直接通过
        }
        tab.setAuditStatus(1);
        auditMapper.updateByPrimaryKeySelective(tab);
        return "";
    }

    @Override
    public void refuse(String auditId, String auditReason, String auditStaffId, String auditName) {
        TabHaobanAudit tab = new TabHaobanAudit();
        tab.setAuditId(auditId);
        tab.setAuditStatus(2);
        tab.setUpdateTime(new Date());
        tab.setAuditReason(auditReason);
        tab.setAuditName(auditName);
        tab.setAuditStaffId(auditStaffId);
        auditMapper.updateByPrimaryKeySelective(tab);
        TabHaobanAudit tab2 = auditMapper.selectByPrimaryKey(auditId);
        int auditType = tab2.getAuditType();
        String oldValue = tab2.getOldValue();
        String enterpriseId = tab2.getEnterpriseId();
        //发送消息
        if (auditType == AuditType.STORE_CHANGE.getCode()) {
            String changeField = tab2.getChangeField();
//			String title = "门店信息修改通知";
            String content = StoreFieldEnum.getValueName(changeField);
//			content = "["+content+"]"+"被拒绝。拒绝理由："+auditReason;
            String staffId = tab2.getCommitStaffId();
            String storeId = tab2.getCommitStoreId();
            this.sendMessageByStaffId(staffId, storeId, true, content, auditReason);


        } else if (auditType == AuditType.CLERK_BIND.getCode()) {
            AuditStaffDTO obj = JSONObject.parseObject(oldValue, AuditStaffDTO.class);
            String staffId = tab2.getCommitStaffId();
            staffDepartmentRelatedApiService.sendBindAuditMessage(auditId, staffId, obj.getClerkId(), auditReason, true);
//			staffDepartmentRelatedApiService.sendStaffBind(obj.getStaffId(), obj.getClerkCode(), tab2.getCommitStoreId(), 2, auditReason);
        } else if (auditType == AuditType.CLERK_ADD.getCode()) {
            AuditStaffDTO obj = JSONObject.parseObject(oldValue, AuditStaffDTO.class);
            staffDepartmentRelatedApiService.sendClerkAdd(obj.getClerkName(), obj.getClerkCode(), tab2.getCommitStoreId(), 2, auditReason);
        } else if (auditType == AuditType.CLERK_DEL.getCode()) {
            AuditStaffDTO obj = JSONObject.parseObject(oldValue, AuditStaffDTO.class);
            staffDepartmentRelatedApiService.sendClerkDel(obj.getClerkName(), obj.getClerkCode(), tab2.getCommitStoreId(), 2, auditReason);
        }
    }

    @Override
    public int save(Integer auditType, String wxEnterpriseId, String enterpriseId,
                    String commitStaffId, String commitStoreId, String changeField,
                    String oldValue, String newValue) {
        TabHaobanStaff staff = staffMapper.selectByPrimaryKey(commitStaffId);
        if (staff == null) {
            return 0;
        }
        TabHaobanAudit tab = new TabHaobanAudit();
        tab.setAuditId(UuidUtil.randomUUID());
        tab.setAuditStatus(0);
        tab.setAuditType(auditType);
        tab.setChangeField(changeField);
        tab.setCommitName(null);
        tab.setCommitStaffId(commitStaffId);
        tab.setCommitStaffImg(staff.getHeadImg());
        tab.setCommitStaffName(staff.getStaffName());
        tab.setCommitStoreId(commitStoreId);
        tab.setCommitTime(new Date());
        tab.setCreateTime(new Date());
        tab.setWxEnterpriseId(wxEnterpriseId);
        tab.setEnterpriseId(enterpriseId);
        tab.setNewValue(newValue);
        tab.setOldValue(oldValue);
        tab.setStatusFlag(1);
        tab.setUpdateTime(new Date());
        return auditMapper.insertSelective(tab);
    }

    @Override
    public void editStoreInfo(String storeId, String changeField,
                              String oldValue, String newValue) {
        logger.info("===================进11");
        StoreDTO store = storeService.getStore(storeId);
        if (!StoreFieldEnum.STORE_IMAG.getValue().equals(changeField)) {
            //非门店图片修改
            logger.info("===================进222");
            StoreDetailDTO detailDTO = new StoreDetailDTO();
            detailDTO = EntityUtil.changeEntityByJSON(StoreDetailDTO.class, store);
            detailDTO.setStoreId(storeId);
            detailDTO.setEnterpriseId(store.getEnterpriseId());
            //detailDTO.setStatus(2);
            detailDTO.setOpenTime(null);
            detailDTO.setCloseTime(null);
            getStoreDTO(detailDTO, changeField, oldValue, newValue);

            logger.info("detailDTO====================>为{}", JSONObject.toJSONString(detailDTO));
            storeService.saveStore(detailDTO);
        } else {
            //门店图片修改
            logger.info("===================进333");
            storeService.updateStorePhotos(storeId, newValue);
        }
    }

    private void getStoreDTO(StoreDetailDTO store, String changeField, String oldValue, String newValue) {
        logger.info("==========================>changeField={}", changeField);
        if (StoreFieldEnum.BUSINESS_TIME_STRING.getValue().equals(changeField)) {
            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
            //营业时间
            String[] newValues = newValue.split("-");
            String openTime = newValues[0];
            String closeTime = newValues[1];
            store.setOpenTime(openTime);
            store.setCloseTime(closeTime);
        } else if (StoreFieldEnum.CONACTS_PHONE.getValue().equals(changeField)) {
            //联系电话
            store.setConactsPhone(newValue);
        } else if (StoreFieldEnum.STORE_ADDRESS.getValue().equals(changeField)) {
            logger.info("==========================>进来333");
            //门店地址
            StoreAddressDTO obj = JSONObject.parseObject(newValue, StoreAddressDTO.class);
            String areaId = obj.getAreaId();
            String storeAddress = obj.getStoreAddress();
            logger.info("==========================>areaId={},storeAddress={},newValue={}", areaId, storeAddress, newValue);
            String[] arr = storeAddress.split(" ");
            store.setAreaId(areaId);
            store.setFullArea(arr[0]);
            //避免多个空格造成数据混乱
            store.setStoreAddress(storeAddress.substring(storeAddress.indexOf(arr[0]) + arr[0].length() + 1));
        } else if (StoreFieldEnum.STORE_AREA.getValue().equals(changeField)) {
            //门店面积
            store.setStoreArea(Double.parseDouble(newValue));
        } else if (StoreFieldEnum.STORE_NAME.getValue().equals(changeField)) {
            //门店名字
            store.setStoreName(newValue);
        } else if (StoreFieldEnum.STORE_IMAG.getValue().equals(changeField)) {
            //门店图片
        }
    }

    @Override
    public List<AuditDTO> listByStoreId(String storeId) {
        List<TabHaobanAudit> list = auditMapper.listByStoreId(storeId);
        List<AuditDTO> resultList = EntityUtil.changeEntityListByJSON(AuditDTO.class, list);
        return resultList;
    }

    @Override
    public AuditDTO findByStoreIdAndChangeField(String storeId,
                                                String changeField) {
        TabHaobanAudit tab = auditMapper.findByStoreIdAndChangeField(storeId, changeField);
        if (tab == null) {
            return null;
        }
        return EntityUtil.changeEntityByJSON(AuditDTO.class, tab);
    }


    private void sendMessageByStaffId(String staffId, String storeId, boolean isRefuse, String content, String reason) {
        StaffClerkRelationDTO staffClerkRelationDTO = staffClerkRelationApiService.getOneByStoreIdAndStaffId(storeId, staffId);
        if (staffClerkRelationDTO == null) {
            logger.info("staff不存在");
            return;
        }

        Map<String, String> params = new HashMap<String, String>();
        params.put("field", content);
        params.put("reason", reason);

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("storeId", storeId);

        if (isRefuse) {
            NoticeMessageUtil.sendNoticeMessage(staffClerkRelationDTO.getEnterpriseId(), staffClerkRelationDTO.getClerkId()
                    , NoticeMessageTypeEnum.STORE_EDIT_REFUSE.getType(), null, params, jsonObject);

        } else {
            NoticeMessageUtil.sendNoticeMessage(staffClerkRelationDTO.getEnterpriseId(), staffClerkRelationDTO.getClerkId()
                    , NoticeMessageTypeEnum.STORE_EDIT_AGREE.getType(), null, params, jsonObject);
        }
    }

    @Override
    public void insert(AuditDTO audit) {
        audit.setAuditId(com.gic.haoban.common.utils.StringUtil.randomUUID());
        audit.setCreateTime(new Date());
        audit.setUpdateTime(new Date());
        auditMapper.insertSelective(EntityUtil.changeEntityByJSON(TabHaobanAudit.class, audit));
    }

    @Override
    public Page<AuditDTO> listByStaffId(String staffId, Integer auditType, Integer auditStatus, BasePageInfo pageInfo) {
        PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
        return PageUtil.changePageHelperToCurrentPage(auditMapper.listByStaffId(staffId, auditType, auditStatus), AuditDTO.class);
    }

    @Override
    public Page<AuditDTO> pageStoreListByParams(String storeId, BasePageInfo pageInfo, Integer auditType,
                                                Integer auditStatus) {
        PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
        return PageUtil.changePageHelperToCurrentPage(auditMapper.pageStoreListByParams(storeId, auditType, auditStatus), AuditDTO.class);
    }

    public void insertBatchLog(String batchId, int auditResult, TabHaobanAudit tab) {
        EnterpriseDTO enterprise = enterpriseService.getEnterpriseById(tab.getEnterpriseId());
        TabHaobanBatchAuditLog batTab = new TabHaobanBatchAuditLog();
        batTab.setAuditId(tab.getAuditId());
        batTab.setBatchAuditLogId(UuidUtil.randomUUID());
        batTab.setAuditType(tab.getAuditType());
        batTab.setAuditResult(auditResult);
        batTab.setBatchId(batchId);
        batTab.setCommitName(tab.getCommitStaffName());
        batTab.setCommitTime(tab.getCommitTime());
        batTab.setCreateTime(new Date());
        batTab.setEnterpriseName(enterprise == null ? "" : enterprise.getEnterpriseName());
        batTab.setStatus(1);
        batTab.setUpdateTime(new Date());
        tabHaobanBatchAuditLogMapper.insert(batTab);
    }

    @Override
    public AuditDTO findById(String auditId) {
        return EntityUtil.changeEntityByJSON(AuditDTO.class, auditMapper.selectByPrimaryKey(auditId));
    }

    @Override
    public void update(AuditDTO audit) {
        audit.setUpdateTime(new Date());
        auditMapper.updateByPrimaryKeySelective(EntityUtil.changeEntityByJSON(TabHaobanAudit.class, audit));
    }

    @Override
    public List<BatchAuditLogDTO> listBatchLog(String batchId) {
        List<TabHaobanBatchAuditLog> list = tabHaobanBatchAuditLogMapper.listByBatchId(batchId);
        return EntityUtil.changeEntityListByJSON(BatchAuditLogDTO.class, list);
    }

    @Override
    public AuditDTO findByBindRelatedIdAndAuditType(String relatedId,
                                                    int auditType) {
        TabHaobanAudit tab = auditMapper.findByBindRelatedIdAndAuditType(relatedId, auditType);
        return EntityUtil.changeEntityByJSON(AuditDTO.class, tab);
    }

    @Override
    public boolean judgeHavePhoneNumberOrCode(String enterpriseId,
                                              String clerkCode, String phoneNumber) {
        List<TabHaobanAudit> list = auditMapper.judgeHavePhoneNumberOrCode(enterpriseId, clerkCode, phoneNumber);
        return !CollectionUtil.isEmpty(list);
    }

}
